% Code Author: Moh Sabbir Saadat
% Institution: University of South Carolina
% Last update date: September 24, 2021
% Website: https://cse.sc.edu/~msaadat/

% this function returns the base profile as well as the idle temperature of an antenna given the tx time
function [temp_fit, idle_temp] = CreateTemperatureProfile(antenna_no)

global folder_currentdissipation
global data_length

% extract the temperature and throughput data from the appropriate files
file = strcat('dissipation_',num2str(antenna_no));
filename = fullfile(folder_currentdissipation, file);
temp_raw = ExtractTemperature(filename);
throughput = ExtractThroughput(filename);

% determine the idle temperature
% and the index of the starting point from raw data
% eliminate the idle period from the raw data
i = 1;
while throughput(i) == 0
    i = i+1;
end
t_start = i; % starting point
idle_temp = mean(temp_raw(1:t_start-1)); % idle temperature
std_idle = std(temp_raw(1:t_start-1));

% determine the index of the end of transmission
while throughput(i) ~= 0
    i = i+1;
end
t_end = i;

% model transmission period and dissipation as exponential growth & decay
% Transmission
temp_raw_tx = temp_raw(t_start-1:t_end-1);
index1 = (t_start-1):(t_end-1);
temp_fit_tx = fit(index1.', temp_raw_tx, 'exp2');

% Dissipation
temp_raw_dissipation = temp_raw(t_end:end);
index2 = t_end:length(temp_raw);
temp_fit_dissipation = fit(index2.', temp_raw_dissipation, 'exp2');

% % you can choose to visualize the fitted temperature profile by
% % uncommenting this block
% %% plot to test
% figure;
% plot(temp_fit_tx, index1, temp_raw_tx)
% hold on
% plot(temp_fit_dissipation, index2, temp_raw_dissipation)
% hold off
% grid on

% turn the fitted model to time series dataset
temp_fit_tx = temp_fit_tx(index1);
temp_fit_dissipation = temp_fit_dissipation(index2);
% concatanate the transmission and dissipation parts
temp_fit = [temp_fit_tx; temp_fit_dissipation];

% negate the idle temperature to create the base profile
temp_fit = temp_fit - idle_temp;

% slice off the trailing end to create data of fixed length
if length(temp_fit) > data_length
    temp_fit(data_length+1:end) = [];
elseif length(temp_fit) < data_length
    while length(temp_fit) ~= data_length
        temp_fit = [temp_fit; NaN];
    end
end

end