% Code Author: Moh Sabbir Saadat
% Institution: University of South Carolina
% Last update date: September 24, 2021
% Website: https://cse.sc.edu/~msaadat/

% This is a simulation of Aquilo: a temperature aware antenna scheduler
% If you use this simulation, please cite the following paper
% M. S. Saadat, S. Sur and S. Nelakuditi,
% "A Case for Temperature-Aware Scheduler for Millimeter-Wave Devices and Networks,"
% 2020 IEEE 28th International Conference on Network Protocols (ICNP), 2020,
% pp. 1-12, doi: 10.1109/ICNP49622.2020.9259381.
% For Bibtex:
% @INPROCEEDINGS{9259381,  author={Saadat, Moh Sabbir and Sur, Sanjib and Nelakuditi, Srihari},
% booktitle={2020 IEEE 28th International Conference on Network Protocols (ICNP)},
% title={A Case for Temperature-Aware Scheduler for Millimeter-Wave Devices and Networks},
%     year={2020},  volume={},  number={},  pages={1-12},  doi={10.1109/ICNP49622.2020.9259381}}

close all; clc;

addOn = matlab.addons.installedAddons;
addOn = join(erase(string(addOn{:, :}), ""), '', 2);
statisticsExist = find(contains(addOn, 'Statistics and Machine Learning Toolbox'));
curveFitExist = find(contains(addOn, 'Curve Fitting Toolbox'));
if isempty(statisticsExist) || isempty(curveFitExist)
    message = sprintf('Cannot execute. The following toolboxes are missing: \n 1. Statistics and Machine Learning Toolbox \n 2. Curve Fitting Toolbox \n');
    error(message)
%     return;
end

clear all; 

% Retrieving global parameters
disp('Retrieving global parameters...')
globalParams;

% Before running Aquilo, we will pre-compute and save
% 1. antenna combination: all the possible antenna combination we can simulate based on the
% number of transmission files we have for each transmission period and the
% number of antennas (no_antenna) we will be simulate in the system
% e.g. if we have 10 transmission files and we intend to simulate 4
% antennas on the system, we can simulate 10C4 = 210 (validate on matlab by running nchoosek(10, 4))
% combinations of antenna on the device;
% 2. for each combination, how many possible antenna schedule is possible
% based on the number of antennas simulated on the system, the number of
% slots (exp_window) in each frame
% e.g. if we simulate 4 antennas in the system, schedule for 10 slots,
% and follow the simple Non-Adjacency criteria to not schedule the last
% scheduled antenna, there will be 4*(3^9) = 78, 732
% 3. channel status: this is a exp_window x no_antenna binary matrix which
% indicates which antenna has an active channel at any given slot. This
% simulates a probabilistic blockage configuration for our scheduler based on a pre-set
% blockage probability (block_prob)
% 4. link lengths: this is a no_antenna x exp_window matrix which specifies the maximum length
% of time each antenna can operate before the transmission must be switched.
% This is computed simulating the time elapsed before a blockage happens as a poisson random
% variable.

% -----------------------------------------------------------------------------%
%% 1. antenna combinations
disp('Computing the possible combinations of antenna...')
dataCombs = nchoosek(1:no_dataFiles, no_antenna);
if ~exist(folder_scheduling, 'dir')
    mkdir(folder_scheduling)
end
writematrix(dataCombs, strcat(folder_scheduling, slash, 'PossibleDataCombinations.csv'));
% -----------------------------------------------------------------------------%

fprintf('%d combination of antenna can be simulated. Please choose any combination.\n (i.e. pick any number between 1 and %d) \n',...
    nchoosek(no_dataFiles, no_antenna), nchoosek(no_dataFiles, no_antenna));
comb_no = input('');
while ~ismember(comb_no, 1:nchoosek(no_dataFiles, no_antenna))
    disp('Please enter a valid integer.')
    comb_no = input('');
end

% -----------------------------------------------------------------------------%
%% 2. antenna schedule list
disp('Computing all the possible schedules...')
schedule_list = [];
ant_set = (1:no_antenna)';
while size(schedule_list, 2) < exp_window
    if isempty(schedule_list)
        schedule_list = ant_set;
    else
        schedule_list = repelem(schedule_list, no_antenna-1, 1);
        schedule_list = [schedule_list, zeros(size(schedule_list, 1), 1)];
        for i = 1:(no_antenna - 1):size(schedule_list, 1)-(no_antenna - 2)
            addendum = ant_set((ant_set ~= schedule_list(i, end-1)));
            schedule_list(i:i+(no_antenna-2), end) = addendum;
        end
    end
end
writematrix(schedule_list, strcat(folder_scheduling, slash, 'PossibleAntennaSchedule.csv'));
% -----------------------------------------------------------------------------%

%% 3. channel status
disp('Simulating the channel conditions of the antennas...')
channel_status = ones(exp_window, no_antenna);
for k = 1:exp_window
    % determine which antennas have good enough channel
    for j = 1:no_antenna
        x = rand;
        channel = (x >= 0.5*block_prob) && (x <= (1- 0.5*block_prob));
        if ~(channel)
            channel_status(k, j) = 0;
        end
    end
end
% check if all the antennas are blocked at a given slot
for k = 1:length(channel_status(:,1))
    avail_ant = channel_status(k,:);
    avail_ant(avail_ant == 0) = [];
    
    while isempty(avail_ant)
        for j = 1:no_antenna
            x = rand;
            channel = (x >= 0.5*block_prob) && (x <= (1- 0.5*block_prob));
            if ~(channel)
                channel_status(k, j) = 0;
            else
                channel_status(k, j) = 1;
            end
        end
        avail_ant = channel_status(k,:);
        avail_ant(avail_ant == 0) = [];
    end
end
% save the blockage configuration
writematrix(schedule_list, strcat(folder_scheduling, slash, 'channel_status.csv'));
% -----------------------------------------------------------------------------%

% -----------------------------------------------------------------------------%
%% 4. link lengths
disp('Simulating link lengths of the antennas...')
link_length = poissrnd(slot_length*100, no_antenna, exp_window);
link_length = link_length/100;
% if any slot length is above tx_max
link_length(link_length > tx_max) = tx_max;
writematrix(link_length, strcat(folder_scheduling, slash, 'link_lengths.csv'));
% -----------------------------------------------------------------------------%

disp('Running Aquilo...')
aquilo;

disp('Evaluate temperature & throughput...')
eval_temp_thr;

% remove temporary directories
% fprintf('Removing temporary directories: %s and %s \n', folder_currentdissipation, folder_scheduling);
%-------------------------------------%
% rmdir(folder_currentdissipation, 's')
% rmdir(folder_scheduling, 's') 
%-------------------------------------%
%-------------------------------------%
% system(sprintf('rmdir /S /Q %s', folder_currentdissipation));
% system(sprintf('rmdir /S /Q %s', folder_scheduling));
%-------------------------------------%

clearvars -except folder_output no_antenna comb_no slash aquilo_schedule throughput_series temp_series throughput_schedule

fprintf('Aquilo schedule: \t');
for i = 1:length(aquilo_schedule)
    fprintf('%d \t', aquilo_schedule(i));
end
fprintf('\n');
figure; 
bar(aquilo_schedule)
yticks(1:no_antenna)
labelsY = {};
for i = 1:no_antenna
    labelsY(i) = strcat('Antenna', {' '}, num2str(i));
end
title('Aquilo schedule')
yticklabels(labelsY)
xlabel('Slot number')

fprintf('Temperature series (in %cC): \n', char(176))
figure; hold on
for i = 1:size(temp_series, 2)
    fprintf('Antenna %d: \t', i);
    for j = 1:size(temp_series, 1)
        fprintf('%.2f \t', temp_series(j, i))
    end
    fprintf('\n')
    plot(temp_series(:, i))
end
title('Temperature series')
xlabel('Slot number')
ylabel(fprintf('Temperature (in %cC)', char(176)))
legend(labelsY)

fprintf('Throughput series (in Mbps): \t');
for i = 1:length(throughput_series)
    fprintf('%.2f \t', throughput_series(i));
end
fprintf('\n');
figure; 
plot(throughput_series/1024)
xlabel('Slot number')
ylabel('Throughput (in Gbps)')
title('Throughput series')

fprintf('Mean throughput (in Mbps): %.2f \n', throughput_schedule);

% Save output
if ~exist(folder_output, 'dir')
    mkdir(folder_output)
end
output_file = strcat(folder_output, slash, 'Aquilo_output_', num2str(comb_no), '.mat');
save(output_file, 'aquilo_schedule', 'throughput_series', 'temp_series', 'throughput_schedule')

